/*
 *
 * Copyright 2013 LinkedIn Corp. All rights reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */
package com.meituan.ptubes.sdk.consumer;

import com.meituan.ptubes.common.log.Logger;
import com.meituan.ptubes.common.log.LoggerFactory;

public class UncaughtExceptionTrackingThread extends Thread implements Thread.UncaughtExceptionHandler {
    private final Logger log;

    public UncaughtExceptionTrackingThread() {
        super();
        setUncaughtExceptionHandler(this);
        this.log = LoggerFactory.getLogger(UncaughtExceptionTrackingThread.class);
    }

    public UncaughtExceptionTrackingThread(Runnable target, String name, String taskName) {
        super(target, name);
        setUncaughtExceptionHandler(this);
        this.log = LoggerFactory.getLoggerByTask(UncaughtExceptionTrackingThread.class, taskName);
    }

    public UncaughtExceptionTrackingThread(Runnable target, String taskName) {
        super(target);
        setUncaughtExceptionHandler(this);
        this.log = LoggerFactory.getLoggerByTask(UncaughtExceptionTrackingThread.class, taskName);
    }

    public UncaughtExceptionTrackingThread(String name, String taskName) {
        super(name);
        setUncaughtExceptionHandler(this);
        this.log = LoggerFactory.getLoggerByTask(UncaughtExceptionTrackingThread.class, taskName);
    }

    private Throwable lastException = null;

    @Override
    public void uncaughtException(Thread t, Throwable e) {
        lastException = e;
        log.error("uncaught exception in thread " + getName() + ": " + e.getMessage(), e);
    }

    public Throwable getLastException() {
        return lastException;
    }

}
